/***************************************************************************
                          ADM_genvideo.hxx  -  description
                             -------------------
    begin                : Tue Mar 19 2002
    copyright            : (C) 2002 by mean
    email                : fixounet@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ADM_VIDEO__
#define __ADM_VIDEO__

#define INT int32_t

void GUI_PreviewInit(uint32_t w , uint32_t h);
uint8_t GUI_PreviewUpdate(uint8_t *data);
void GUI_PreviewEnd( void );
uint8_t GUI_StillAlive( void );

#define Pixel uint8_t
typedef struct
 {
 		uint32_t width;
 		uint32_t height;
 		uint32_t nb_frames;
 		uint32_t encoding;
 		uint32_t codec;
 		uint32_t fps1000;
 }ADV_Info;



typedef struct
{
    uint32_t					w,h;
    uint32_t				algo;
}RESIZE_PARAMS;

typedef struct
{
 	uint32_t cropx,cropx2;
 	uint32_t cropy,cropy2;
}CROP_PARAMS;

typedef struct {
    int pixel;
    double weight;
} CONTRIB;


typedef struct {
    uint32_t  n;			/* number of contributors */
    CONTRIB *p;			/* pointer to list of contributions */
} CLIST;

typedef struct
{
	uint32_t 	width;		/* horizontal size of the image in Pixels */
	uint32_t	height;		/* vertical size of the image in Pixels */
	Pixel *	data;		/* pointer to first scanline of image */

} Image;



 class AVDMGenericVideoStream
 {
   protected:
      		ADV_Info 	_info;		
   		    uint8_t   *_uncompressed;
          uint8_t		getPixel(int32_t x,int32_t y,uint8_t *data);
          uint8_t		getPixelU(int32_t x,int32_t y,uint8_t *data);
          uint8_t		setPixelU(uint8_t val,int32_t x,int32_t y,uint8_t *data);

          uint8_t 	unPackChroma(uint8_t *ssrc,uint8_t *ddst);

   public:
          virtual 	void *configure( AVDMGenericVideoStream *instream)=0;

          				AVDMGenericVideoStream( void)  {};
		           
  				virtual ~AVDMGenericVideoStream( ) {};
          virtual char *printConf(void) { static char *str=(char *)"."; return str;};
  				
          virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
          																				uint8_t *data,uint32_t *flags)=0;
          ADV_Info 				*getInfo( void ) { return &_info;};		
          virtual uint8_t	*getBinaryConf( uint16_t *size) { *size=0;return NULL;}

 };

 // Pseudo class used to register filters automagically
 // Does not work atm
class AVDMVideo_FilterDec
{
private:
	int dum;
public:
  	AVDMVideo_FilterDec(char *name, AVDMGenericVideoStream *(*create) (AVDMGenericVideoStream *in, void *));  	
};



class  AVDMVideoStreamNull :public AVDMGenericVideoStream
 {

 protected:
    		ADM_Composer 	*_in;
    		uint8_t				*_uncompressed;
    		
 public:
 		
  				AVDMVideoStreamNull(  ADM_Composer *in);  	          							
  				virtual ~AVDMVideoStreamNull();
  				
  				
          virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
          																	uint8_t *data,uint32_t *flags);
         	

           virtual 	void *configure( AVDMGenericVideoStream *instream);


 }     ;


class  AVDMVideoStreamCrop:public AVDMGenericVideoStream
 {

 protected:
    		AVDMGenericVideoStream 	*_in;
    		uint8_t									*_uncompressed;
    		CROP_PARAMS          *_param;
    		
 public:
 		
  				AVDMVideoStreamCrop(  AVDMGenericVideoStream *in,void *setup);  	          							
  	virtual ~AVDMVideoStreamCrop();
          virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
          																	uint8_t *data,uint32_t *flags);
          			   virtual char *printConf(void) ;														
					void *configure( AVDMGenericVideoStream *instream);          																	
          virtual uint8_t	*getBinaryConf( uint16_t *size) { *size=sizeof(CROP_PARAMS); return (uint8_t *)_param;}

 }     ;
 class  AVDMVideoStreamBSMear:public AVDMGenericVideoStream
 {

 protected:
    		AVDMGenericVideoStream 	*_in;
    		uint8_t									*_uncompressed;
    		CROP_PARAMS          *_param;
    		
 public:
 		
  				AVDMVideoStreamBSMear(  AVDMGenericVideoStream *in,void *setup);  	          							
  	virtual ~AVDMVideoStreamBSMear();
          virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
          																	uint8_t *data,uint32_t *flags);
          			   virtual char *printConf(void) ;														
					void *configure( AVDMGenericVideoStream *instream);          																	
          virtual uint8_t	*getBinaryConf( uint16_t *size) { *size=sizeof(CROP_PARAMS);return (uint8_t *)_param;}

 }     ;

class  AVDMVideoStreamResize:public AVDMGenericVideoStream
 {

 protected:
    		AVDMGenericVideoStream 	*_in;
    		uint8_t									*_uncompressed;
    		RESIZE_PARAMS          *_param;
    		uint8_t								_init;	
           INT *Hpattern_luma;
           INT *Hpattern_chroma;
           INT *Vpattern_luma;
           INT *Vpattern_chroma;
           uint8_t *_intermediate_buffer;
		// engine
		void ResizeV(Image * iin, Image * iout, INT *pattern) ;
		void ResizeVFIR4(Image * iin, Image * iout, INT *pattern) ;
		
		void ResizeH(Image * iin, Image * iout, INT *pattern) ;
 		void ResizeHFIR4(Image * iin, Image * iout, INT  *pattern)	;
   
 		void precompute(Image * dst, Image * src, uint8_t algo);
           uint8_t 	zoom(Image * dst, Image * src);
           void endcompute(void);

 public:
 		
  				AVDMVideoStreamResize(  AVDMGenericVideoStream *in,void *setup);  	          							
  				virtual 		~AVDMVideoStreamResize();
          virtual 		uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
          																	uint8_t *data,uint32_t *flags);
					void *configure( AVDMGenericVideoStream *instream);          																	
					   virtual char *printConf(void) ;
          		
          virtual uint8_t	*getBinaryConf( uint16_t *size) { *size=sizeof(RESIZE_PARAMS);return (uint8_t *) _param;}



 }     ;
AVDMGenericVideoStream *filterCreateFromTag(uint32_t tag,uint8_t *conf,AVDMGenericVideoStream *in);

#endif

